import os
from abcli import env, file, path, string
from abcli.plugins.storage import instance as storage
from abcli.logger import logger


def list_of_files(object_name, cloud=False, **kwargs):
    """return list of files in object_name.

    Args:
        object_name (str): object name.
        cloud (bool, optional): on cloud. Defaults to False.

    Returns:
        List[str]: list of files.
    """
    return (
        storage.list_of_objects(
            object_name,
            **kwargs,
        )
        if cloud
        else file.list_of(
            os.path.join(
                env.abcli_object_root,
                object_name,
                "*",
            ),
            **kwargs,
        )
    )


def object_path(
    object_name=".",
    create=False,
):
    output = os.path.join(
        env.abcli_object_root,
        env.abcli_object_name if object_name == "." else object_name,
    )

    if create:
        os.makedirs(output, exist_ok=True)

    return output


def path_of(
    filename,
    object_name=".",
    create=False,
):
    return os.path.join(
        object_path(object_name, create),
        filename,
    )


def select(object_name: str):
    os.environ["abcli_object_name"] = object_name
    env.abcli_object_name = object_name

    path = object_path(object_name)
    os.environ["abcli_object_path"] = path
    env.abcli_object_path = path


def signature(info=None, object_name="."):
    return [
        "{}{}".format(
            env.abcli_object_name if object_name == "." else object_name,
            "" if info is None else f"/{str(info)}",
        ),
        string.pretty_date(include_time=False),
        string.pretty_date(include_date=False, include_zone=True),
    ]


def unique_object(prefix=""):
    object_name = string.pretty_date(
        as_filename=True,
        unique=True,
    )
    if prefix:
        object_name = f"{prefix}-{object_name}"

    path.create(object_path(object_name))

    logger.info(f"📂 {object_name}")

    return object_name



import argparse
from . import *
from abcli.logger import logger
from blueness.argparse.generic import sys_exit


parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    default="get",
    help="clone|create|get|search|set",
)
parser.add_argument(
    "--after",
    type=str,
    default="",
    help="123-4-e",
)
parser.add_argument(
    "--before",
    type=str,
    default="",
)
parser.add_argument(
    "--count",
    type=int,
    default=-1,
)
parser.add_argument(
    "--offset",
    type=int,
    default=0,
)
parser.add_argument(
    "--delim",
    type=str,
    default=", ",
)
parser.add_argument(
    "--host",
    default=-1,
    type=int,
    help="0|1|-1",
)
parser.add_argument(
    "--item_name",
    default="object",
    type=str,
)
parser.add_argument(
    "--log",
    default=1,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--object",
    type=str,
    default="",
)
parser.add_argument(
    "--object_2",
    type=str,
    default="",
)
parser.add_argument(
    "--shuffle",
    default=0,
    type=int,
    help="0|1",
)
parser.add_argument(
    "--tag",
    type=str,
    default="",
)
parser.add_argument(
    "--tags",
    type=str,
    default="",
    help="tag_1,~tag_2",
)
parser.add_argument(
    "--type",
    type=str,
    default="",
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = False
output = None
if args.task == "clone":
    success = clone(args.object, args.object_2)
elif args.task == "create":
    success = create()
elif args.task == "get":
    output = get(args.object)
    success = True
elif args.task == "search":
    output = search(
        args.tags,
        after=args.after,
        before=args.before,
        count=args.count,
        host=args.host,
        shuffle=args.shuffle,
        offset=args.offset,
    )
    success = True
elif args.task == "set":
    success = set_(args.object, args.tags)
else:
    success = None

if success is True and output is not None:
    if args.log:
        logger.info(f"{len(output):,} {args.item_name}(s): {delim.join(output)}")
    else:
        print(delim.join(output))

sys_exit(logger, NAME, args.task, success, log=args.log)



import re
import glob
import os
from abcli import file
from abcli.plugins.git import NAME
from abcli.logger import logger

NAME = f"{NAME}.version"


def increment(
    repo_path: str,
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.increment({repo_path})")

    list_of_paths = sorted(
        [
            file.path(filename)
            for filename in glob.glob(
                os.path.join(repo_path, "**", "__init__.py"),
                recursive=True,
            )
        ]
    )
    if verbose:
        logger.info(
            "{} options: {}".format(
                len(list_of_paths),
                ", ".join(list_of_paths),
            )
        )

    if not list_of_paths:
        logger.error("cannot find __init__.py, quitting.")
        return False

    filename = os.path.join(list_of_paths[0], "__init__.py")
    success, source_code = file.load_text(filename)
    if not success:
        return success

    version_pattern = r'^(VERSION\s*=\s*")(\d+\.\d+\.\d+)(")$'
    updated_source_code = []
    for line in source_code:
        match = re.match(version_pattern, line)
        if match:
            version_parts = match.group(2).split(".")
            version_parts[1] = str(int(version_parts[1]) + 1)
            updated_line = match.group(1) + ".".join(version_parts) + match.group(3)

            logger.info(f"{line} -> {updated_line}")

            updated_source_code.append(updated_line)
        else:
            updated_source_code.append(line)

    while updated_source_code and not updated_source_code[len(updated_source_code) - 1]:
        updated_source_code = updated_source_code[:-1]

    return file.save_text(filename, updated_source_code, log=True)



from abcli import env
from abcli.modules.objects import object_path
from abcli.modules.host import shell


def download_object(object_name: str) -> bool:
    return shell(
        "aws s3 sync {}/{}/ {}".format(
            env.abcli_s3_object_prefix,
            object_name,
            object_path(object_name, create=True),
        )
    )



import argparse
import sys
from abcli import string, NAME
from abcli import file
from abcli.logger import logger
from blueness.argparse.generic import sys_exit

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="replace|size",
)
parser.add_argument(
    "--filename",
    type=str,
)
parser.add_argument(
    "--this",
    type=str,
)
parser.add_argument(
    "--that",
    type=str,
)
args = parser.parse_args()

if args.task in ["replace"]:
    if not file.exist(args.filename):
        logger.error(f"-{NAME}: {args.task}: {args.filename}: file not found.")
        sys.exit(1)

success = False
if args.task == "replace":
    logger.info(f"{NAME}.{args.task}: {args.this} -> {args.that} in {args.filename}")

    success, content = file.load_text(args.filename)
    if success:
        content = [line.replace(args.this, args.that) for line in content]

        success = file.save_text(args.filename, content)
elif args.task == "size":
    print(string.pretty_bytes(file.size(args.filename)))
    success = True
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from functools import reduce
from abcli import file


def sort(filename):
    """sort help file - used by "abcli help".

    Args:
        filename (str): filename.

    Returns:
        bool: success.
    """
    success, content = file.load_text(filename)
    if not success:
        return success

    info = {
        keyword.lstrip(): value for keyword, value in zip(content[::2], content[1::2])
    }

    return file.save_text(
        filename,
        reduce(
            lambda x, y: x + y,
            [[key, info[key]] for key in sorted(info.keys())],
            [],
        ),
    )



import cv2
from functools import reduce
import numpy as np
from . import NAME
from abcli.logger import logger


def add_label(
    image,
    x,
    y,
    label,
    background=0,
    font_color=3 * (127,),
    font_face=cv2.FONT_HERSHEY_SIMPLEX,
    font_scale=0.5,
    line_type=cv2.LINE_AA,
    thickness=1,
):
    """add label to image.

    Args:
        image (np.ndarray): image.
        x (int): x.
        y (int): y,
        label (str): label
        background (int, optional): background color. Defaults to 0.
        font_color (tuple, optional): font color. Defaults to 3*(127,).
        font_face (int, optional): font face. Defaults to cv2.FONT_HERSHEY_SIMPLEX.
        font_scale (float, optional): font scale. Defaults to 0.5.
        line_type (int, optional): line type. Defaults to cv2.LINE_AA.
        thickness (int, optional): _description_. Defaults to 1.

    Returns:
        np.ndarray: image.
    """

    text_size = cv2.getTextSize(
        text=label,
        fontFace=font_face,
        fontScale=font_scale,
        thickness=thickness,
    )
    text_height = text_size[0][1] + text_size[1] + 2
    text_width = text_size[0][0]

    try:
        image[y - text_height : y, x : x + text_width] = background
    except:
        pass

    cv2.putText(
        image,
        text=label,
        org=[x - 1, y - text_size[1]],
        fontFace=font_face,
        fontScale=font_scale,
        lineType=line_type,
        color=font_color,
        thickness=thickness,
    )

    return image


def render_text(
    text,
    box=False,
    centered=False,
    color_depth=3,
    font_color=3 * (127,),
    font_face=cv2.FONT_HERSHEY_SIMPLEX,
    font_size=2,
    height=1.0,
    image=None,
    image_width=None,
    left=0.0,
    line_type=cv2.LINE_AA,
    thickness=1,
    top=0.0,
    width=1.0,
):
    """render text

    Args:
        text (List(str)): text.
        box (bool, optional): draw a box around text. Defaults to False.
        centered (bool, optional): centered. Defaults to False.
        color_depth (int, optional): color depth. Defaults to 3.
        font_color (_type_, optional): font color. Defaults to 3*(127,).
        font_face (_type_, optional): font face. Defaults to cv2.FONT_HERSHEY_SIMPLEX.
        font_size (int, optional): font size. Defaults to 2.
        height (float, optional): height. normalized to image size.. Defaults to 1.0.
        image (Any, optional): image. Default to None.
        image_width (Any, optional): image width - applicable if image is None. Default to None.
        left (float, optional): left. normalized to image size.. Defaults to 0.0.
        line_type (_type_, optional): line type. Defaults to cv2.LINE_AA.
        thickness (int, optional): thickness. Defaults to 1.
        top (float, optional): top. normalized to image size. Defaults to 0.0.
        width (float, optional): width. normalized to image size.. Defaults to 1.0.

    Returns:
        np.ndarray: image.
    """
    if image is None:
        if image_width is None:
            logger.error(f"-{NAME}: render_text(None): image_width is missing.")
            return image
    else:
        if not image.shape or len(image.shape) < 2:
            return image

    if not isinstance(font_color, list) and not isinstance(font_color, tuple):
        font_color = 3 * (font_color,)

    if not isinstance(text, list):
        text = [text]

    # https://stackoverflow.com/a/51285918/10917551
    # (label_width, label_height), baseline
    text_line_size = [
        cv2.getTextSize(
            text=text_line,
            fontFace=font_face,
            fontScale=font_size,
            thickness=thickness,
        )
        for text_line in text
    ]

    required_text_height = reduce(
        lambda x, y: x + y,
        [
            text_line_size_[0][1] + text_line_size_[1] + 2
            for text_line_size_ in text_line_size
        ],
        0,
    )
    required_text_width = max(
        [text_line_size_[0][0] for text_line_size_ in text_line_size]
    )

    if image is None:
        text_width = int(image_width * width)

        font_size *= text_width / required_text_width
    else:
        text_height = int(image.shape[0] * height)
        text_width = int(image.shape[1] * width)

        font_size *= min(
            [text_width / required_text_width, text_height / required_text_height]
        )

    # (label_width, label_height), baseline
    text_line_size = [
        cv2.getTextSize(
            text=text_line,
            fontFace=font_face,
            fontScale=font_size,
            thickness=thickness,
        )
        for text_line in text
    ]

    required_text_height = reduce(
        lambda x, y: x + y,
        [
            text_line_size_[0][1] + text_line_size_[1] + 2
            for text_line_size_ in text_line_size
        ],
        0,
    )

    if image is None:
        image = np.zeros(
            (required_text_height, image_width, color_depth),
            dtype=np.uint8,
        )

    text_height = int(image.shape[0] * height)
    text_width = int(image.shape[1] * width)

    line_spacing = (text_height - required_text_height) / (1 + len(text))

    text_left = int(image.shape[1] * left)
    text_top = int(image.shape[0] * top)

    x = text_left
    y = text_top + line_spacing
    for index, text_line in enumerate(text):
        y += text_line_size[index][0][1] + 2

        actual_x = int(
            x + text_width / 2 - text_line_size[index][0][0] / 2 if centered else x
        )

        cv2.putText(
            image,
            text=text_line,
            org=(
                actual_x,
                int(y),
            ),
            fontFace=font_face,
            fontScale=font_size,
            color=font_color,
            lineType=line_type,
            thickness=thickness,
        )
        if box:
            cv2.rectangle(
                image,
                (
                    int(actual_x),
                    int(y - text_line_size[index][0][1]),
                ),
                (
                    int(actual_x + text_line_size[index][0][0]),
                    int(y + text_line_size[index][1]),
                ),
                color=font_color,
                thickness=thickness // 2,
                lineType=line_type,
            )

        y += line_spacing

    if box:
        cv2.rectangle(
            image,
            (text_left, text_top),
            (text_left + text_width, text_top + text_height),
            color=font_color,
            thickness=thickness,
            lineType=line_type,
        )

    return image
